from flask import Flask,redirect,abort,send_from_directory,safe_join
import json
import os
import re

#our stuff
from .Config import Config
from .Collection import CollectionManager

class OrgBlogApp:
    
    def __init__(self, config=Config()):
        self.__config = config
        self.__flask = Flask(__name__,
                           static_folder=self.__config.lookup("static_directory"),
                           static_url_path="/static")

        self.__collection_manager = CollectionManager(self.__config)
        
        @self.__flask.route("/", methods=["GET"])
        def route_root():
            index_template = os.path.join(self.__config.lookup("template_directory"), "index.html")
            
            if not os.path.exists(index_template) or not os.path.isfile(index_template):
                abort(404)
            else:
                return self.get_all_collections(format="html")

        @self.__flask.route("/<string:collection_slug>/", methods=["GET"])
        @self.__flask.route("/<string:collection_slug>", methods=["GET"])
        def route_get_items(collection_slug):
            try:
                html = self.get_all_collection_items(collection_slug, format="html")
                return html
            except LookupError:
                abort(404)
                
                
        @self.__flask.route("/<string:collection_slug>/<string:item_slug>", methods=["GET"])
        def route_get_item(collection_slug, item_slug):
            try:
                html = self.get_collection_item(collection_slug, item_slug, format="html")
                return html
            except LookupError:
                abort(404)

        @self.__flask.route("/<string:collection_slug>/tag/<string:tag_slug>", methods=["GET"])
        def route_get_items_with_tag(collection_slug, tag_slug):
            try:
                html = self.get_all_collection_items_with_tag(collection_slug, tag_slug, format="html")
                return html
            except LookupError:
                abort(404)        
                
    def run(self):
        self.__flask.run(host=self.__config.lookup('host'), port=int(self.__config.lookup('port')))

    def generate(self):
        dest = os.path.join(os.getcwd(), "_site")
        print("generating static site into '{0}'".format(dest))
        
        #index page
        if not os.path.exists(dest):
            os.makedirs(dest)
            
        with open(os.path.join(dest, "index.html"), "w") as f:
            f.write(self.get_all_collections(format="html"))

        for collection in self.get_all_collections():
            #collection index pages
            collection_directory = os.path.join(dest, collection.get_slug())
            if not os.path.exists(collection_directory):
                os.makedirs(collection_directory)
            with open(os.path.join(collection_directory, "index.html"), "w") as f:
                f.write(self.get_all_collection_items(collection.get_slug(), format="html"))

            #collection tag pages
            tag_directory = os.path.join(collection_directory, "tag")
            if not os.path.exists(tag_directory):
                os.makedirs(tag_directory)
            for tag in collection.get_tags():
                with open(os.path.join(tag_directory, tag + ".html"), "w") as f:
                    f.write(self.get_all_collection_items_with_tag(collection.get_slug(), tag, format="html"))

            #collection item pages
            for item in collection.get_items():
                with open(os.path.join(collection_directory, item['slug'] + ".html"), "w") as f:
                    f.write(self.get_collection_item(collection.get_slug(), item['slug'], format="html"))

    #CollectionManager convenience methods
    def get_slugs(self):
        return self.__collection_manager.get_slugs()

    def get_all_collections(self, *args, **kwargs):
        return self.__collection_manager.get_all_collections(**kwargs)

    def get_all_collection_items(self, collection_slug, *, format):
        return self.__collection_manager.get_all_collection_items(collection_slug, format=format)

    def get_collection_item(self, collection_slug, item_slug, *, format):
        return self.__collection_manager.get_collection_item(collection_slug, item_slug, format=format)

    def get_all_collection_items_with_tag(self, collection_slug, tag_slug, *, format):
        return self.__collection_manager.get_all_collection_items_with_tag(collection_slug, tag_slug, format=format)



from Org import *

op = OrgParser()
entries = op.load("/home/cam/git/org-blog.git/org-files/posts.org")

def show_entry(e,level):
    indent = ""
    for n in range(1,level):
        indent += "\t"
        
    print(indent + "title:")
    print(indent + e.get_title())

    print(indent + "level:")
    print(indent + str(e.get_level()))

    print(indent + "tags:")
    print(indent + str(e.get_tags()))

    # for s in e.get_sections():
    #     print(indent + "------------------------")
    #     print(indent + str(s))

    print(indent + "subheadings:")
    for entry in e.get_subentries():
        show_entry(entry,level + 1)

    print(indent + "html:")
    print(indent + e.to_html())

    print (indent + "---------------")


for e in entries:
    show_entry(e,1)



from setuptools import setup

setup(name='orgblog',
      version='0.7.6',
      description='easy web publishing with org-mode files',
      url='http://github.com/ctindall/org-blog',
      author='Cameron Tindall',
      author_email="cam@tindall.space",
      license='GPL',
      packages=['orgblog'],
      zip_safe=False,
      scripts=['bin/orgblog'],
      install_requires = [
          "flask",
          "pystache"
      ])



import re
import string

class OrgParseError(Exception):
    pass

class OrgSection:
    def __init__(self, type="regular_text"):
        self.__type = type
        self.__lines = []

    def __str__(self):
        return "section type: " + self.__type + "\n-----------------" + "\n".join(self.__lines)
    
    def set_type(self, type):
        self.__type = type

    def get_type(self):
        return self.__type

    def add_line(self, line):
        self.__lines.append(line)

    def add_lines(self, lines):
        self.__lines = self.__lines + lines
    
    def get_lines(self):
        return self.__lines

class OrgEntry:
    def __init__(self, title=""):
        self.__title = title
        self.__text = ""
        self.__tags = []
        self.__subentries = []
        self.__superentries = []
        self.__sections = []
        self.__todo_status = ""
        
    #title
    def set_title(self, title):
        self.__title = title

    def get_title(self):
        return self.__title

    #level
    def set_level(self, level):
        self.__level = level

    def get_level(self):
        return self.__level

    #tags
    def add_tag(self, tag):
        self.__tags.append(tag)
        
    def add_tags(self, tags):
        self.__tags = self.__tags + tags
        
    def get_tags(self):
        tags = self.__tags
        for se in self.get_superentries():
            tags = tags + se.get_tags()
            
        return tags

    #TODO status
    def set_todo_status(self, status):
        self.__todo_status = status

    def get_todo_status(self):
        return self.__todo_status
    
    #superentries
    def add_superentry(self, entry):
        self.__superentries.append(entry)

    def get_superentries(self):
        return self.__superentries
    
    #subentries
    def add_subentry(self, entry):
        self.__subentries.append(entry)
        entry.add_superentry(self)

    def get_subentries(self):
        return self.__subentries

    #sections
    def add_section(self, section):
        self.__sections.append(section)

    def get_sections(self):
        return self.__sections
    
    #text
    def add_line(self, line):
        self.__text = self.__text + "\n" + line

    def get_text(self):
        return self.__text

    def __inline_quotify(self, line):
        regex = '\=(.*?)\='

        match = re.search(regex, line)
        while match:
            text = match.group(1)
            html = "<code>" + text + "</code>"
            line = line.replace(match.group(), html)
            match = re.search(regex, line)

        return line

    def __boldify(self, line):
        regex = '\*(.*?)\*'

        match = re.search(regex, line)
        while match:
            text = match.group(1)
            html = "<b>" + text + "</b>"
            line = line.replace(match.group(), html)
            match = re.search(regex, line)

        return line
    
    def __linkify(self, line):
        #return the same line, but with org-mode link syntax turned into HTML links
        with_link_text_regex = '(\[\[)(https|http|ftps|ftp)(://)(.*?)(\]\[)(.*?)(\]\])'
        without_link_text_regex = '(\[\[)(https|http|ftps|ftp)(://)(.*?)(\]\[)(.*?)'
        
        match_with_link_text = re.search(with_link_text_regex, line)
        match_without_link_text = re.search(without_link_text_regex, line)
        while match_with_link_text or match_without_link_text:
            if match_with_link_text:
                match = match_with_link_text
            else:
                match = match_without_link_text
            
            proto = match.group(2)
            url = match.group(4)
            if match.group(6):
                link_text = match.group(6)
            else:
                link_text = url
                                    
            link = "<a href='" + proto + "://" + url + "'>" + link_text + "</a>"
            line = line.replace(match.group(), link)
            match_with_link_text = re.search(with_link_text_regex, line)
            match_without_link_text = re.search(without_link_text_regex, line)


        return line
    
    def to_html(self):
        heading_tags = {
            1: "h1",
            2: "h2",
            3: "h3",
            4: "h4",
            5: "h5",
            6: "h6"
        }

        if self.get_level() > 6:
            heading_tag = "b"
        else:
            heading_tag = heading_tags[self.get_level()]
        
        html = "<div class='entry'>\n"

        if self.get_level() > 1: #don't include top-level titles as these are handled by the template
            html += "<" + heading_tag + ">" + self.get_title() + "</" + heading_tag + ">\n"

        for section in self.get_sections():
            html += "<div class='section-" + section.get_type() + "'>\n"
            for line in section.get_lines():
                line = self.__inline_quotify(line)
                line = self.__boldify(line)
#                line = self.__italicize(line)
                line = self.__linkify(line)
                    
                html += line + "\n"
            html += "</div>"
            
        html += "<p>" + self.get_text() + "\n</p>\n"
        for subentry in self.get_subentries():
            html += subentry.to_html()
        html += "</div>\n"
        
        return html

class LineStream:
    def __init__(self, string):
        self.__lines = string.splitlines()
        self.__position = 0

    def peek(self): #look at current line in stream
        if self.__position < len(self.__lines):
            return self.__lines[self.__position]
        else:
            return False

    def finished(self):
        if self.__position >= len(self.__lines):
            return True
        else:
            return False

        
    def eat(self): #advance the tape to the next line
        self.__position = self.__position + 1

    def get_line_number(self):
        return self.__position + 1
    
class OrgParser:

    def __init__(self):
        self.__settings = {}
        self.set_setting("TODO", "TODO | DONE") #default TODO states

    def get_valid_todo_statuses(self):
        words = self.get_setting("TODO").split()
        return [word for word in words if word != "|"]
        
    def get_setting(self, setting_name):
        return self.__settings[setting_name]

    def get_all_settings(self):
        return self.__settings
    
    def set_setting(self, setting_name, setting_value):
        self.__settings[setting_name] = setting_value

    def load(self, filename):
        return self.loads(open(filename).read())

    def loads(self, string):
        self.__stream = LineStream(string)
        return self.__parse_entry_list()

    def __eat(self):
        self.__stream.eat()

    def __peek(self):
        return self.__stream.peek()
    
    def __parse_heading_line(self, level=1):
        if self.__peek():
            line_without_tags = re.sub('(\s+)?(\:.*\:)(\s+)?$', "", self.__peek())
            match = re.search('^(\*+)(\s)([A-Z][A-Z][A-Z]+)?(\s+)?(.*)(\:.*\:)?', line_without_tags)
            if not match:
                raise OrgParseError("trouble parsing heading at line " + str(self.__stream.get_line_number()))

            todo_status = match.group(3)
            parsed_level = len(match.group(1))
            space_between_todo_and_title = match.group(4)
            title = match.group(5)

            if todo_status not in self.get_valid_todo_statuses() or not todo_status:
                # If the parsed TODO status is not actually a valid
                # status, then we were mistaken and the todo_status is
                # really part of the title. Add it back, along with
                # the white space that we thought separated them.
                todo_status = ""
                
                if not title:
                    title = ""

                if not space_between_todo_and_title:
                    space_between_todo_and_title = ""

            if match and parsed_level == level:
                return title, level, todo_status
            
        return None, None, None
    
    def __parse_regular_line(self):
        if self.__peek():
            is_heading = re.search('^[\*]', self.__peek())
            is_option  = re.search('^(\s+)?\#\+', self.__peek())
            if not is_heading  and not is_option:
                return self.__peek()
            
        return None

    def __parse_tags(self):
        match = re.search('(\:.*\:)', self.__peek())
        if match:
        #if we're on a heading line and it has a tag portion...
        #split the tag portion on ":" characters and get rid of the empty tags
            parsed_tags = list(filter(lambda t: t != '', match.group().split(":")))
        else:
            parsed_tags = []
        return parsed_tags

    def __parse_regular_text_section(self):
        lines = []
        
        regular_line = self.__parse_regular_line()
        while regular_line:
            lines.append(regular_line)
            self.__eat()
            regular_line = self.__parse_regular_line()

        if lines != []:
            section = OrgSection()
            section.add_lines(lines)
            return section
        else:
            return None
        
    def __parse_source_section(self):
        lines = []
        
        is_src = self.__parse_begin_src()
        if is_src:
            self.__eat() #down with the BEGIN_SRC line
            
            while not self.__parse_end_src():
                lines.append(self.__peek())
                self.__eat()

            self.__eat() #done with the END_SRC line

            #we're done, create a section and return it
            section = OrgSection(type="source")
            section.add_lines(lines)
            return section
        else:
            return None
    
    def __parse_begin_src(self):
        if self.__peek():
            match = re.search('^(\s+)?(\#\+BEGIN_SRC)', self.__peek())
            if match:
                return True

        return None

    def __parse_end_src(self):
        if self.__peek():
            match = re.search('^(\s+)?(\#\+END_SRC)', self.__peek())
            if match:
                return True

        return None        

    def __parse_blank_line(self):
        if not self.__stream.finished(): #if there are lines left
            match = re.search('^\s*$', self.__peek())
            if match: #...and the line is blank
                return True
            
        return False
                            
    def __eat_blank_lines(self):
        blank_line = self.__parse_blank_line()
        while blank_line:
            self.__eat()
            blank_line = self.__parse_blank_line()
        
    def __parse_section(self):
        self.__eat_blank_lines()
        section = self.__parse_regular_text_section() or self.__parse_source_section()
        return section

    def __parse_settings_line(self):
        if self.__peek():
            match = re.search('^\#\+([^\:]+)\:(.*)$', self.__peek())
            if match:
                option = match.group(1)
                value = match.group(2)

                #we're not looking for #+BEGIN_* and #+END_* lines, since those are handled elsewhere 
                if re.search('^(BEGIN|END)_', option):
                    return False, False
                
                return option, value

        return False, False
    
    def __parse_settings_lines(self):
        option, value = self.__parse_settings_line()
        while option and value:
            self.set_setting(option, value)
            self.__eat()
            self.__eat_blank_lines()
            option, value = self.__parse_settings_line()
            
    def __parse_entry(self, level=1):
        self.__eat_blank_lines()

        self.__parse_settings_lines()
        
        my_title, my_level, my_todo_status = self.__parse_heading_line(level=level);
        if my_title and my_level == level:
            #start a new entry
            entry = OrgEntry()

            entry.set_title(my_title)
            entry.set_level(my_level)
            entry.set_todo_status(my_todo_status)
            entry.add_tags(self.__parse_tags())
            self.__stream.eat()             #we're done with the heading line

            #look for sections
            section = self.__parse_section()
            while section:
                entry.add_section(section)
                section = self.__parse_section()
            
            #recursively look for any subheadings of the next higher level
            subentry = self.__parse_entry(level=my_level + 1)
            while subentry:
                entry.add_subentry(subentry)
                subentry = self.__parse_entry(level=my_level + 1)

            return entry
        else:
            return None    

        
    def __parse_entry_list(self):
        global re
        entries = []

        entry = self.__parse_entry()
        while entry:
            entries.append(entry)
            entry = self.__parse_entry()

        return entries



from .Org import OrgParser
from .Config import Config

import pystache
import re
import os

class CollectionManager():

    def __init__(self, config=Config()):
        self.__collections_directory = config.lookup('collections_directory')
        self.__template_directory = config.lookup('template_directory')
        
        self.__collections = [];
        
        #find all the relevant collections in the collections directory
        for f in os.scandir(self.__collections_directory):

            #create a Collection object for each relevant file and append it to our list
            match = re.search('^([^\#.]+)(\.org)$', os.path.basename(f.name))
            if match:
                slug = match.group(1)
                collection = Collection(slug, config)
                self.__collections.append(collection)

    def __update_context(self):
        self.__context = {}
        for collection in self.__collections:
            self.__context[collection.get_slug()] = collection.get_items()
        
    def get_slugs(self):
        return list(map(self.get_all_collections(),
                        lambda col: col.get_slug()))
    
    def get_all_collections(self, format=None):
        self.__update_context()
        
        if not format:
            return self.__collections

        if format == "html":
            template = open(os.path.join(self.__template_directory, "index.html"), "r").read()
            return pystache.render(template, self.__context)
        
        else:
            return self.__context

    def __items_template_exists(self, collection_slug):
        template_path = os.path.join(self.__template_directory, collection_slug, "items.html")
        return os.path.exists(template_path)
        
    def get_all_collection_items(self, collection_slug, format=None):
        
        if format == "html" and not self.__items_template_exists(collection_slug):
            raise LookupError("There is no items.html template for the collection '" + collection_slug + "'")
        
        for c in self.get_all_collections():
            if c.get_slug() == collection_slug:
                return c.get_items(format=format)
            
        raise LookupError("The collection '" + collection_slug + "' does not exist")

    def __item_template_exists(self, collection_slug):
        template_path = os.path.join(self.__template_directory, collection_slug, "item.html")
        return os.path.exists(template_path)
    
    def get_collection_item(self, collection_slug, item_slug, format=None):
        if format == "html" and not self.__item_template_exists(collection_slug):
            raise LookupError("There is no item.html template for the collection '" + collection_slug + "'")
        
        for c in self.get_all_collections():
            if c.get_slug() == collection_slug:
                return c.get_item(item_slug, format=format)

    def __tag_template_exists(self, collection_slug):
        template_path = os.path.join(self.__template_directory, collection_slug, "tag.html")
        return os.path.exists(template_path)

    def get_all_collection_items_with_tag(self, collection_slug, tag_slug, format=None):
        if format == "html" and not self.__tag_template_exists(collection_slug):
            raise LookupError("There is no tag.html template for the collection '" + collection_slug + "'")

        for c in self.get_all_collections():
            if c.get_slug() == collection_slug:
                return c.get_items_with_tag(tag_slug, format=format)
            
        raise LookupError("The collection '" + collection_slug + "' does not exist")
       
        
class Collection:
    
    def __init__(self, collection_slug, config=Config()):
        self.__orgfile_path = os.path.join(config.lookup('collections_directory'), collection_slug + ".org")
        self.__template_directory = config.lookup('template_directory')
        self.__collection_slug = collection_slug
        self.__visible_todo_statuses = config.lookup("visible_todo_statuses")
        self.__tags = set([])

    def __slugify(self, slug):
        slug = re.sub(r"[^\w\s-]", '', slug)#remove weird characters
        slug = re.sub(r"\s+", '-', slug)#condense whitespace down to dashes
        slug = slug.lower()
        return slug

    def __updateContext(self):
        self.context = {}
        self.context['items'] = []

        op = OrgParser()
        items = op.load(self.__orgfile_path)
        for i in items:
            if i.get_todo_status() in self.__visible_todo_statuses:
                # The template context should exclude unfinished
                # posts. By default, we only show those with the
                # status 'DONE'.
                item = {}
                item['title'] = i.get_title()
                item['slug'] = self.__slugify(item['title'])
                item['html'] = i.to_html()
                item['tags'] = i.get_tags()
                item['todo_status'] = i.get_todo_status()

                self.context['items'].append(item)
                self.__tags.update(item['tags'])

    def get_tags(self):
        self.__updateContext()
        return self.__tags
                
    def get_items(self, format=None):
        self.__updateContext()
        
        if(format == "html"):
            template = open(os.path.join(self.__template_directory, self.__collection_slug, "items.html"), "r").read()
            return pystache.render(template, self.context)
            
        else:
            return self.context['items']
        
    def get_item(self, slug, format=None):
        self.__updateContext()
        item = None
        
        for i in self.context['items']:
            if i['slug'] == slug:
                item = i

        if not item:
            raise LookupError("The item '" + slug + "' does not exist in the collection '" + self.__collection_slug + "'")
        
        if(format == "html"):
            template = open(os.path.join(self.__template_directory, self.__collection_slug, "item.html"), "r").read()
            return pystache.render(template, item)
        else:
            return item

    def get_items_with_tag(self, tag, format=None):
        self.__updateContext()
        items = []
        
        for i in self.context['items']:
            if tag in i['tags']:
                items.append(i)

        context = {}
        context['tag'] = tag
        context['items'] = items
        
        if(format == "html"):
            template = open(os.path.join(self.__template_directory, self.__collection_slug, "tag.html"), "r").read()
            return pystache.render(template, context)
        else:
            return items

        
    def get_slug(self):
        return self.__collection_slug



import json
import os


class Config:

    def __init__(self, **kwargs):
        defaults = { 
            "collections_directory": "~/.orgblog/collections/",
            "template_directory": "~/.orgblog/templates/",
            "static_directory": "~/.orgblog/static/",
            "visible_todo_statuses": ["DONE"],
            "host": "127.0.0.1",
            "port": "5000"
        }
        
        self.__config = {}

        if 'filename' in kwargs.keys() and os.path.isfile(kwargs['filename']):
            self.__config = json.loads(open(kwargs['filename']).read())
        elif os.path.isfile(os.path.expanduser("~/.orgblog.json")):
            self.__config = json.loads(open(os.path.expanduser("~/.orgblog.json")).read())
        elif os.path.isfile("/etc/orgblog.json"):
            self.__config = json.loads(open("/etc/orgblog.json").read())
        else:
            self.__config = defaults

        for key in defaults:
            if not key in self.__config:
                self.__config[key] = defaults[key]

            if key.endswith("_directory"):
                self.__config[key] = os.path.expanduser(self.__config[key])


    def read(self):
        return self.__config

    def lookup(self, key):
        return self.__config[key]

    def install_default(self, silent=False):
        if os.path.isdir(os.path.expanduser("~/")):
            conf_file = open(os.path.expanduser("~/.orgblog.json"), "w")
        else:
            return False

        try:
            if not silent:
                print("Installing default config to ~/.orgblog.json")

            json.dump(defaults, conf_file)
            conf_file.close()
            return True
    
        except FileNotFoundEror:
            return False
